from typing import (
    Iterator,
    Literal,
    Optional,
    TypeVar,
    Generic,
    KeysView,
    ValuesView,
    ItemsView,
)

V = TypeVar("V")

class Entity:
    @staticmethod
    def from_wdentity_json(data: bytes) -> Entity: ...
    @property
    def id(self) -> str: ...
    def entity_type(self) -> Literal["item", "property"]: ...
    @property
    def label(self) -> str: ...
    def label_default_lang(self) -> str: ...
    def lang_in_lang(self, lang: str) -> bool: ...
    def label_keys(self) -> KeysView[str]: ...
    def label_values(self) -> ValuesView[str]: ...
    def label_items(self) -> ItemsView[str, str]: ...
    def description(self) -> str: ...
    def description_default_lang(self) -> str: ...
    def description_keys(self) -> KeysView[str]: ...
    def description_values(self) -> ValuesView[str]: ...
    def description_items(self) -> ItemsView[str, str]: ...
    def aliases(self) -> ListView[str]: ...
    def aliases_default_lang(self) -> str: ...
    def aliases_keys(self) -> KeysView[str]: ...
    def aliases_values(self) -> ValuesView[ListView[str]]: ...
    def aliases_items(self) -> ItemsView[str, ListView[str]]: ...
    def props_keys(self) -> KeysView[str]: ...
    def props_values(self) -> ValuesView[str]: ...
    def props_items(self) -> ItemsView[str, ListView[StatementView]]: ...
    def prop(self, id: str) -> ListView[StatementView]: ...

class StatementView:
    def value(self) -> ValueView: ...
    def qualifiers_keys(self) -> KeysView[str]: ...
    def qualifiers_values(self) -> ValuesView[ListView[ValueView]]: ...
    def qualifiers_items(self) -> ItemsView[str, ListView[ValueView]]: ...
    def qualifiers_order(self) -> ListView[str]: ...
    def qualifier(self, id: str) -> ListView[ValueView]: ...
    def rank(self) -> Literal["normal", "preferred", "deprecated"]: ...

class ValueView:
    def get_type(
        self,
    ) -> Literal[
        "string", "entity-id", "quantity", "time", "globe-coordinate", "monolingualtext"
    ]: ...
    def is_str(self) -> bool: ...
    def is_entity_id(self) -> bool: ...
    def is_quantity(self) -> bool: ...
    def is_time(self) -> bool: ...
    def is_globe_coordinate(self) -> bool: ...
    def is_monolingual_text(self) -> bool: ...
    def as_str(self) -> str: ...
    def as_entity_id_str(self) -> str: ...
    def as_entity_id(self) -> EntityId: ...
    def as_quantity(self) -> Quantity: ...
    def as_time(self) -> Time: ...
    def as_globe_coordinate(self) -> GlobeCoordinate: ...
    def as_monolingual_text(self) -> MonolingualText: ...
    def to_string_repr(self) -> str: ...

class Value:
    @staticmethod
    def string(value: str) -> Value: ...
    @staticmethod
    def entity_id(
        id: str, entity_type: Literal["item", "property"], numeric_id: Optional[int]
    ) -> Value: ...
    @staticmethod
    def time(
        time: str,
        timezone: int,
        before: int,
        after: int,
        precision: int,
        calendarmodel: str,
    ) -> Value: ...
    @staticmethod
    def quantity(
        amount: str,
        lower_bound: Optional[str],
        upper_bound: Optional[str],
        unit: str,
    ) -> Value: ...
    @staticmethod
    def globe_coordinate(
        latitude: float,
        longitude: float,
        precision: Optional[float],
        altitude: Optional[float],
        globe: str,
    ) -> Value: ...
    @staticmethod
    def monolingual_text(text: str, language: str) -> Value: ...
    def get_type(
        self,
    ) -> Literal[
        "string", "entity-id", "quantity", "time", "globe-coordinate", "monolingualtext"
    ]: ...
    def is_str(self) -> bool: ...
    def is_entity_id(self) -> bool: ...
    def is_quantity(self) -> bool: ...
    def is_time(self) -> bool: ...
    def is_globe_coordinate(self) -> bool: ...
    def is_monolingual_text(self) -> bool: ...
    def as_str(self) -> str: ...
    def as_entity_id_str(self) -> str: ...
    def as_entity_id(self) -> EntityId: ...
    def as_quantity(self) -> Quantity: ...
    def as_time(self) -> Time: ...
    def as_globe_coordinate(self) -> GlobeCoordinate: ...
    def as_monolingual_text(self) -> MonolingualText: ...
    def to_string_repr(self) -> str: ...

class EntityId:
    id: str
    entity_type: Literal["item", "property"]
    numeric_id: Optional[int]

class Time:
    time: str
    timezone: int
    before: int
    after: int
    precision: int
    calendarmodel: str

class Quantity:
    amount: str
    lower_bound: Optional[str]
    upper_bound: Optional[str]
    unit: str

class GlobeCoordinate:
    latitude: float
    longitude: float
    precision: float
    altitude: Optional[float]
    globe: str

class MonolingualText:
    text: str
    language: str

class ListView(Iterator[V], Generic[V]):
    """A convenience class that is an iterator with random access. Note that when the iterator
    is exhausted, you won't be able to iterate over it again.
    For example:
    >>> l = ListView([1, 2, 3])  # demonstrate only as you won't be able to initialize it from Python
    >>> l[0]
    1
    >>> list(l)
    [1, 2, 3]
    >>> list(l)
    []
    >>> l[0]
    1
    """

    def __len__(self) -> int: ...
    def __getitem__(self, index: int) -> V: ...
